@@ -32,7 +32,7 @@
 #include <camel/camel.h>
 #include <libedataserverui/e-passwords.h>
 #include <libedataserver/e-account.h>
-#include <libecal/e-cal.h>
+#include <libecal/e-cal-client.h>
 #include <libedataserver/e-account-list.h>
 #include <libedataserver/e-source.h>
 #include <libedataserver/e-source-list.h>
@@ -45,9 +45,20 @@
 
 #define d(x)
 
+#define SET_KRB_SSO(_esource, _krbval) \
+	G_STMT_START { \
+		ESource *tmp_src = (_esource); \
+		const gchar *tmp_val = (_krbval); \
+		e_source_set_property (tmp_src, "kerberos", tmp_val); \
+		if (tmp_val && g_str_equal (tmp_val, "required")) { \
+			e_source_set_property (tmp_src, "auth", NULL); \
+			e_source_set_property (tmp_src, "auth-type", NULL); \
+		} \
+	} G_STMT_END
+
 G_DEFINE_TYPE (ExchangeMAPIAccountListener, exchange_mapi_account_listener, G_TYPE_OBJECT)
 
-static gboolean create_profile_entry (CamelURL *url, EAccount *account);
+static gboolean create_profile_entry (CamelURL *url, EAccount *account, CamelMapiSettings *settings);
 
 struct _ExchangeMAPIAccountListenerPrivate {
 	GConfClient *gconf_client;
@@ -215,8 +226,9 @@ find_source_by_fid (GSList *sources, const gchar *fid)
 #define ADDRESSBOOK_SOURCES     "/apps/evolution/addressbook/sources"
 
 static void
-add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType folder_type, CamelURL *url, mapi_id_t trash_fid)
+add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType folder_type, CamelURL *url, CamelSettings *settings, mapi_id_t trash_fid)
 {
+	CamelMapiSettings *mapi_settings;
 	ESourceList *source_list = NULL;
 	ESourceGroup *group = NULL;
 	const gchar *conf_key = NULL, *source_selection_key = NULL;
@@ -224,6 +236,18 @@ add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType fold
 	GSList *ids, *temp_list, *old_sources = NULL;
 	gchar *base_uri = NULL;
 	gboolean is_new_group = FALSE;
+	const gchar *profile;
+	const gchar *domain;
+	const gchar *realm;
+	const gchar *kerberos;
+	gboolean stay_synchronized;
+
+	mapi_settings = CAMEL_MAPI_SETTINGS (settings);
+	profile = camel_mapi_settings_get_profile (mapi_settings);
+	domain = camel_mapi_settings_get_domain (mapi_settings);
+	realm = camel_mapi_settings_get_realm (mapi_settings);
+	kerberos = camel_mapi_settings_get_kerberos (mapi_settings) ? "required" : NULL;
+	stay_synchronized = camel_offline_settings_get_stay_synchronized (CAMEL_OFFLINE_SETTINGS (settings));
 
 	if (folder_type == MAPI_FOLDER_TYPE_APPOINTMENT) {
 		conf_key = CALENDAR_SOURCES;
@@ -260,8 +284,10 @@ add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType fold
 	e_source_group_set_property (group, "create_source", "yes");
 	e_source_group_set_property (group, "username", url->user);
 	e_source_group_set_property (group, "host", url->host);
-	e_source_group_set_property (group, "profile", camel_url_get_param (url, "profile"));
-	e_source_group_set_property (group, "domain", camel_url_get_param (url, "domain"));
+	e_source_group_set_property (group, "profile", profile);
+	e_source_group_set_property (group, "domain", domain);
+	e_source_group_set_property (group, "realm", realm);
+	e_source_group_set_property (group, "kerberos", kerberos);
 
 	/* We set these because on new folder creation - these are required. */
 	e_source_group_set_property (group, "acl-user-name", account->id->name);
@@ -292,18 +318,18 @@ add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType fold
 			is_new_source = TRUE;
 		}
 		e_source_set_property (source, "auth", "1");
-		e_source_set_property (source, "auth-domain", EXCHANGE_MAPI_PASSWORD_COMPONENT);
 		e_source_set_property (source, "auth-type", "plain/password");
 		e_source_set_property (source, "username", url->user);
 		e_source_set_property (source, "host", url->host);
-		e_source_set_property (source, "profile", camel_url_get_param (url, "profile"));
-		e_source_set_property (source, "domain", camel_url_get_param (url, "domain"));
+		e_source_set_property (source, "profile", profile);
+		e_source_set_property (source, "domain", domain);
+		e_source_set_property (source, "realm", realm);
 		e_source_set_property (source, "folder-id", fid);
 		e_source_set_property (source, "public", "no");
+		SET_KRB_SSO(source, kerberos);
 
 		if (is_new_source)
-			e_source_set_property (source, "offline_sync",
-					camel_url_get_param (url, "offline_sync") ? "1" : "0");
+			e_source_set_property (source, "offline_sync", stay_synchronized ? "1" : "0");
 
 		if (folder->is_default)
 			e_source_set_property (source, "delete", "no");
@@ -324,6 +350,9 @@ add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType fold
 		e_source_set_property (source, "acl-owner-name", account->id->name);
 		e_source_set_property (source, "acl-owner-email", account->id->address);
 
+		if (folder_type != MAPI_FOLDER_TYPE_APPOINTMENT)
+			e_source_set_property (source, "alarm", "never");
+
 		if (is_new_source)
 			e_source_group_add_source (group, source, -1);
 
@@ -368,9 +397,10 @@ add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType fold
 
 void exchange_mapi_add_esource (CamelURL *url, const gchar *folder_name, const gchar *fid, gint folder_type)
 {
+	CamelMapiSettings *settings;
 	ESourceList *source_list = NULL;
 	ESourceGroup *group = NULL;
-	const gchar *conf_key = NULL;
+	const gchar *conf_key = NULL, *kerberos = NULL;
 	GConfClient* client;
 	GSList *sources;
 	ESource *source = NULL;
@@ -413,23 +443,29 @@ void exchange_mapi_add_esource (CamelURL *url, const gchar *folder_name, const g
 		}
 	}
 
+	settings = g_object_new (CAMEL_TYPE_MAPI_SETTINGS, NULL);
+	camel_settings_load_from_url (CAMEL_SETTINGS (settings), url);
+
 	relative_uri = g_strconcat (";", fid, NULL);
+	kerberos = camel_mapi_settings_get_kerberos (settings) ? "required" : NULL;
 	source = e_source_new (folder_name, relative_uri);
 	e_source_set_property (source, "auth", "1");
-	e_source_set_property (source, "auth-domain", EXCHANGE_MAPI_PASSWORD_COMPONENT);
 	e_source_set_property (source, "auth-type", "plain/password");
 	e_source_set_property (source, "username", url->user);
 	e_source_set_property (source, "host", url->host);
-	e_source_set_property (source, "profile", camel_url_get_param (url, "profile"));
-	e_source_set_property (source, "domain", camel_url_get_param (url, "domain"));
+	e_source_set_property (source, "profile", camel_mapi_settings_get_profile (settings));
+	e_source_set_property (source, "domain", camel_mapi_settings_get_domain (settings));
+	e_source_set_property (source, "realm", camel_mapi_settings_get_realm (settings));
 	e_source_set_property (source, "folder-id", fid);
-	e_source_set_property (source, "offline_sync",
-				camel_url_get_param (url, "offline_sync") ? "1" : "0");
+	e_source_set_property (source, "offline_sync", camel_offline_settings_get_stay_synchronized (CAMEL_OFFLINE_SETTINGS (settings)) ? "1" : "0");
 	e_source_set_property (source, "public", "yes");
 	e_source_set_property (source, "delete", "yes");
+	SET_KRB_SSO(source, kerberos);
 
 	e_source_group_add_source (group, source, -1);
 
+	g_object_unref (settings);
+
 	g_object_unref (source);
 	g_free (relative_uri);
 
@@ -568,10 +604,16 @@ add_calendar_sources (EAccount *account, GSList *folders, mapi_id_t trash_fid)
 	url = camel_url_new (account->source->url, NULL);
 
 	if (url) {
-		add_cal_esource (account, folders, MAPI_FOLDER_TYPE_APPOINTMENT, url, trash_fid);
-		add_cal_esource (account, folders, MAPI_FOLDER_TYPE_TASK, url, trash_fid);
-		add_cal_esource (account, folders, MAPI_FOLDER_TYPE_MEMO, url, trash_fid);
+		CamelSettings *settings;
 
+		settings = g_object_new (CAMEL_TYPE_MAPI_SETTINGS, NULL);
+		camel_settings_load_from_url (settings, url);
+
+		add_cal_esource (account, folders, MAPI_FOLDER_TYPE_APPOINTMENT, url, settings, trash_fid);
+		add_cal_esource (account, folders, MAPI_FOLDER_TYPE_TASK, url, settings, trash_fid);
+		add_cal_esource (account, folders, MAPI_FOLDER_TYPE_MEMO, url, settings, trash_fid);
+
+		g_object_unref (settings);
 		camel_url_free (url);
 	}
 }
@@ -619,6 +661,7 @@ static gboolean
 add_addressbook_sources (EAccount *account, GSList *folders, mapi_id_t trash_fid)
 {
 	CamelURL *url;
+	CamelMapiSettings *settings;
 	ESourceList *list;
 	ESourceGroup *group;
 	ESource *source;
@@ -626,12 +669,26 @@ add_addressbook_sources (EAccount *account, GSList *folders, mapi_id_t trash_fid
 	GSList *temp_list, *old_sources = NULL;
 	GConfClient* client;
 	gboolean is_new_group = FALSE;
+	const gchar *profile;
+	const gchar *domain;
+	const gchar *realm;
+	const gchar *kerberos;
+	gboolean stay_synchronized;
 
 	url = camel_url_new (account->source->url, NULL);
 	if (url == NULL) {
 		return FALSE;
 	}
 
+	settings = g_object_new (CAMEL_TYPE_MAPI_SETTINGS, NULL);
+	camel_settings_load_from_url (CAMEL_SETTINGS (settings), url);
+
+	profile = camel_mapi_settings_get_profile (settings);
+	domain = camel_mapi_settings_get_domain (settings);
+	realm = camel_mapi_settings_get_realm (settings);
+	kerberos = camel_mapi_settings_get_kerberos (settings) ? "required" : NULL;
+	stay_synchronized = camel_offline_settings_get_stay_synchronized (CAMEL_OFFLINE_SETTINGS (settings));
+
 	base_uri = g_strdup_printf ("mapi://%s@%s/", url->user, url->host);
 	client = gconf_client_get_default ();
 	list = e_source_list_new_for_gconf (client, "/apps/evolution/addressbook/sources" );
@@ -652,8 +709,10 @@ add_addressbook_sources (EAccount *account, GSList *folders, mapi_id_t trash_fid
 	e_source_group_set_property (group, "user", NULL);
 	e_source_group_set_property (group, "username", url->user);
 	e_source_group_set_property (group, "host", url->host);
-	e_source_group_set_property (group, "profile", camel_url_get_param (url, "profile"));
-	e_source_group_set_property (group, "domain", camel_url_get_param (url, "domain"));
+	e_source_group_set_property (group, "profile", profile);
+	e_source_group_set_property (group, "domain", domain);
+	e_source_group_set_property (group, "realm", realm);
+	e_source_group_set_property (group, "kerberos", kerberos);
 
 	for (temp_list = folders; temp_list != NULL; temp_list = g_slist_next (temp_list)) {
 		ExchangeMAPIFolder *folder = temp_list->data;
@@ -677,18 +736,18 @@ add_addressbook_sources (EAccount *account, GSList *folders, mapi_id_t trash_fid
 			is_new_source = TRUE;
 		}
 		e_source_set_property (source, "auth", "plain/password");
-		e_source_set_property (source, "auth-domain", EXCHANGE_MAPI_PASSWORD_COMPONENT);
 		e_source_set_property(source, "user", NULL);
 		e_source_set_property(source, "username", url->user);
 		e_source_set_property(source, "host", url->host);
-		e_source_set_property(source, "profile", camel_url_get_param (url, "profile"));
-		e_source_set_property(source, "domain", camel_url_get_param (url, "domain"));
+		e_source_set_property(source, "profile", profile);
+		e_source_set_property(source, "domain", domain);
+		e_source_set_property(source, "realm", realm);
 		e_source_set_property(source, "folder-id", fid);
 		e_source_set_property (source, "public", "no");
+		SET_KRB_SSO(source, kerberos);
 
 		if (is_new_source) {
-			e_source_set_property (source, "offline_sync",
-					       camel_url_get_param (url, "offline_sync") ? "1" : "0");
+			e_source_set_property (source, "offline_sync", stay_synchronized ? "1" : "0");
 			e_source_set_property (source, "completion", "true");
 		}
 
@@ -739,15 +798,14 @@ add_addressbook_sources (EAccount *account, GSList *folders, mapi_id_t trash_fid
 		}
 		g_free (uri);
 		e_source_set_property (source, "auth", "plain/password");
-		e_source_set_property (source, "auth-domain", "MAPIGAL");
 
 		//FIXME: Offline handling
 		e_source_set_property(source, "user", NULL);
 		e_source_set_property(source, "username", url->user);
-		e_source_set_property(source, "host", camel_url_get_param (url, "ad_server"));
-		e_source_set_property(source, "view-limit", camel_url_get_param (url, "ad_limit"));
-		e_source_set_property(source, "profile", camel_url_get_param (url, "profile"));
-		e_source_set_property(source, "domain", camel_url_get_param (url, "domain"));
+		e_source_set_property(source, "profile", profile);
+		e_source_set_property(source, "domain", domain);
+		e_source_set_property(source, "realm", realm);
+		SET_KRB_SSO(source, kerberos);
 
 		if (is_new_source) {
 			e_source_set_property(source, "offline_sync", "1");
@@ -780,6 +838,8 @@ add_addressbook_sources (EAccount *account, GSList *folders, mapi_id_t trash_fid
 	if (!e_source_list_sync (list, NULL))
 		g_warning ("%s: Failed to sync source list", G_STRFUNC);
 
+	g_object_unref (settings);
+
 	g_object_unref (group);
 	g_object_unref (list);
 	g_object_unref (client);
@@ -985,23 +1045,31 @@ static void
 update_account_sources_async (gpointer worker_data, gboolean cancelled, gpointer user_data)
 {
 	CamelURL *url;
+	CamelSettings *settings;
 	ExchangeMapiConnection *conn;
 	EAccount *account = worker_data;
 	gboolean can_create_profile = GPOINTER_TO_INT (user_data) ? TRUE : FALSE;
+	const gchar *profile;
 
 	url = camel_url_new (account->source->url, NULL);
 	g_return_if_fail (url != NULL);
 
-	conn = exchange_mapi_connection_find (camel_url_get_param (url, "profile"));
+	settings = g_object_new (CAMEL_TYPE_MAPI_SETTINGS, NULL);
+	camel_settings_load_from_url (settings, url);
+
+	profile = camel_mapi_settings_get_profile (CAMEL_MAPI_SETTINGS (settings));
+
+	conn = exchange_mapi_connection_find (profile);
 	if (!conn && can_create_profile) {
 		/* connect to the server when not connected yet */
-		if (!create_profile_entry (url, account)) {
+		if (!create_profile_entry (url, account, CAMEL_MAPI_SETTINGS (settings))) {
 			camel_url_free (url);
+			g_object_unref (settings);
 			g_warning ("%s: Failed to create MAPI profile for '%s'", G_STRFUNC, account->name);
 			return;
 		}
 
-		conn = exchange_mapi_connection_find (camel_url_get_param (url, "profile"));
+		conn = exchange_mapi_connection_find (profile);
 	}
 
 	if (conn) {
@@ -1010,7 +1078,7 @@ update_account_sources_async (gpointer worker_data, gboolean cancelled, gpointer
 		struct create_sources_data *csd;
 
 		csd = g_new0 (struct create_sources_data, 1);
-		csd->profile_name = g_strdup (camel_url_get_param (url, "profile"));
+		csd->profile_name = g_strdup (profile);
 		csd->account = g_object_ref (account);
 
 		g_timeout_add_seconds (1, check_for_account_conn_cb, csd);
@@ -1018,6 +1086,7 @@ update_account_sources_async (gpointer worker_data, gboolean cancelled, gpointer
 
 	camel_url_free (url);
 	g_object_unref (account);
+	g_object_unref (settings);
 }
 
 static void
@@ -1086,7 +1155,7 @@ mapi_account_removed (EAccountList *account_listener, EAccount *account)
 		GError *error = NULL;
 
 		exchange_mapi_delete_profile (profile, &error);
-		e_passwords_forget_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
+		e_passwords_forget_password (NULL, key);
 
 		g_free (key);
 		camel_url_free (url);
@@ -1101,46 +1170,51 @@ mapi_account_removed (EAccountList *account_listener, EAccount *account)
 }
 
 static gboolean
-create_profile_entry (CamelURL *url, EAccount *account)
+create_profile_entry (CamelURL *url, EAccount *account, CamelMapiSettings *settings)
 {
 	gboolean status = FALSE;
 	guint8 attempts = 0;
+	ExchangeMapiProfileData empd = { 0 };
 
 	if (!e_shell_get_online (e_shell_get_default ()))
 		return FALSE;
 
+	empd.server = url->host;
+	empd.username = url->user;
+	exchange_mapi_util_profiledata_from_settings (&empd, settings);
+
 	while (!status && attempts <= 3) {
-		gchar *password = NULL, *key = NULL;
+		gchar *key = NULL;
 
 		key = camel_url_to_string (url, CAMEL_URL_HIDE_PASSWORD | CAMEL_URL_HIDE_PARAMS);
-		if (!attempts)
-			password = e_passwords_get_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
-		if (!password) {
+		if (!attempts && !empd.krb_sso)
+			empd.password = e_passwords_get_password (NULL, key);
+		if (!empd.password && !empd.krb_sso) {
 			gboolean remember = account && e_account_get_bool (account, E_ACCOUNT_SOURCE_SAVE_PASSWD);
 			gchar *title;
 
-			title = g_strdup_printf (_("Enter Password for %s@%s"), url->user, url->host);
-			password = e_passwords_ask_password (title, EXCHANGE_MAPI_PASSWORD_COMPONENT, key, title,
-					E_PASSWORDS_REMEMBER_FOREVER | E_PASSWORDS_SECRET | (attempts ? E_PASSWORDS_REPROMPT : 0),
-					&remember, NULL);
+			title = g_strdup_printf (_("Enter Password for %s@%s"),
+						 url->user, url->host);
+			empd.password = e_passwords_ask_password (title, NULL, key, title, E_PASSWORDS_REMEMBER_FOREVER | E_PASSWORDS_SECRET | (attempts ? E_PASSWORDS_REPROMPT : 0), &remember, NULL);
 			g_free (title);
 		}
 		g_free (key);
 
-		if (password) {
-			GError *error = NULL;
-			guint32 cp_flags = (camel_url_get_param (url, "ssl") && g_str_equal (camel_url_get_param (url, "ssl"), "1")) ? CREATE_PROFILE_FLAG_USE_SSL : CREATE_PROFILE_FLAG_NONE;
 
-			status = exchange_mapi_create_profile (url->user, password, camel_url_get_param (url, "domain"), url->host, cp_flags, NULL, NULL, &error);
+		if (empd.password || empd.krb_sso) {
+			GError *error = NULL;
+
+			status = exchange_mapi_create_profile (&empd, NULL,
+							       NULL, &error);
 			if (status) {
 				/* profile was created, try to connect to the server */
 				ExchangeMapiConnection *conn;
 				gchar *profname;
 
 				status = FALSE;
-				profname = exchange_mapi_util_profile_name (url->user, camel_url_get_param (url, "domain"), url->host, FALSE);
+				profname = exchange_mapi_util_profile_name (&empd, FALSE);
 
-				conn = exchange_mapi_connection_new (profname, password, &error);
+				conn = exchange_mapi_connection_new (profname, empd.password, &error);
 				if (conn) {
 					status = exchange_mapi_connection_connected (conn);
 					g_object_unref (conn);
@@ -1173,7 +1247,7 @@ check_equal (const gchar *a, const gchar *b)
 static gboolean
 mapi_camel_url_equal (CamelURL *a, CamelURL *b)
 {
-	const gchar *params[] = { "profile", "domain", "ad_limit", "ad_server" };
+	const gchar *params[] = { "profile", "domain", "realm", "kerberos" };
 	guint n_params = G_N_ELEMENTS (params), i;
 	gboolean retval = TRUE;
 
@@ -1195,6 +1269,8 @@ mapi_account_changed_async (gpointer worker_data, gboolean cancelled, gpointer u
 	ExchangeMAPIAccountInfo *existing_account_info = NULL;
 	EAccountList *account_listener = worker_data;
 	EAccount *account = user_data;
+	ExchangeMapiProfileData empd = { 0 };
+	CamelSettings *settings;
 
 	g_return_if_fail (account_listener != NULL);
 	g_return_if_fail (account != NULL);
@@ -1207,17 +1283,28 @@ mapi_account_changed_async (gpointer worker_data, gboolean cancelled, gpointer u
 	if (existing_account_info)
 		old_url = camel_url_new (existing_account_info->source_url, NULL);
 
+	if (!isa_mapi_account && !existing_account_info)
+		return;
+
 	new_url = camel_url_new (account->source->url, NULL);
 
+	settings = g_object_new (CAMEL_TYPE_MAPI_SETTINGS, NULL);
+	camel_settings_load_from_url (settings, new_url);
+
 	if (existing_account_info == NULL && isa_mapi_account) {
 		/* some account of other type is changed to MAPI */
-		if (create_profile_entry (new_url, account)) {
+		if (create_profile_entry (new_url, account, CAMEL_MAPI_SETTINGS (settings))) {
 			/* Things are successful */
 			gchar *profname = NULL, *uri = NULL;
 			ExchangeMAPIAccountListener *config_listener = exchange_mapi_accounts_peek_config_listener();
 
-			profname = exchange_mapi_util_profile_name (new_url->user, camel_url_get_param (new_url, "domain"), new_url->host, FALSE);
-			camel_url_set_param(new_url, "profile", profname);
+			empd.server = new_url->host;
+			empd.username = new_url->user;
+			exchange_mapi_util_profiledata_from_settings (&empd, CAMEL_MAPI_SETTINGS (settings));
+			profname = exchange_mapi_util_profile_name (&empd,
+								    FALSE);
+			camel_mapi_settings_set_profile (CAMEL_MAPI_SETTINGS (settings), profname);
+			camel_settings_save_to_url (settings, new_url);
 			g_free (profname);
 
 			uri = camel_url_to_string(new_url, 0);
@@ -1244,13 +1331,17 @@ mapi_account_changed_async (gpointer worker_data, gboolean cancelled, gpointer u
 		} else if (!mapi_camel_url_equal (old_url, new_url) || (existing_account_info->enabled != account->enabled)) {
 		/* Some or all of the account info changed OR the account has been moved from a disabled state to enabled state */
 			mapi_account_removed (account_listener, account);
-			if (create_profile_entry (new_url, account)) {
+			if (create_profile_entry (new_url, account, CAMEL_MAPI_SETTINGS (settings))) {
 				/* Things are successful */
 				gchar *profname = NULL, *uri = NULL;
 				ExchangeMAPIAccountListener *config_listener = exchange_mapi_accounts_peek_config_listener();
 
-				profname = exchange_mapi_util_profile_name (new_url->user, camel_url_get_param (new_url, "domain"), new_url->host, FALSE);
-				camel_url_set_param(new_url, "profile", profname);
+				empd.server = new_url->host;
+				empd.username = new_url->user;
+				exchange_mapi_util_profiledata_from_settings (&empd, CAMEL_MAPI_SETTINGS (settings));
+				profname = exchange_mapi_util_profile_name (&empd, FALSE);
+				camel_mapi_settings_set_profile (CAMEL_MAPI_SETTINGS (settings), profname);
+				camel_settings_save_to_url (settings, new_url);
 				g_free (profname);
 
 				uri = camel_url_to_string(new_url, 0);
@@ -1271,6 +1362,7 @@ mapi_account_changed_async (gpointer worker_data, gboolean cancelled, gpointer u
 	if (old_url)
 		camel_url_free (old_url);
 
+	g_object_unref (settings);
 	camel_url_free (new_url);
 
 	g_object_unref (account_listener);
@@ -1313,7 +1405,7 @@ exchange_mapi_account_listener_construct (ExchangeMAPIAccountListener *config_li
 		}
 	}
 
-	d(g_debug ("MAPI listener is constructed with %d listed MAPI accounts ", g_list_length (mapi_accounts)));
+	d(exchange_mapi_debug_print ("MAPI listener is constructed with %d listed MAPI accounts ", g_list_length (mapi_accounts)));
 
 	g_signal_connect (config_listener->priv->account_list, "account_added", G_CALLBACK (mapi_account_added), NULL);
 	g_signal_connect (config_listener->priv->account_list, "account_changed", G_CALLBACK (mapi_account_changed), NULL);
