@@ -60,7 +60,7 @@ gboolean
 fetch_props_to_mail_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 {
 	long *flags = NULL;
-	struct FILETIME *delivery_date = NULL;
+	struct FILETIME *delivery_date = NULL, *submit_time = NULL;
 	const gchar *msg_class = NULL, *content_class = NULL;
 	ExchangeMAPIStream *body = NULL;
 	uint32_t content_class_pid;
@@ -73,7 +73,7 @@ fetch_props_to_mail_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 	g_return_val_if_fail (data != NULL, FALSE);
 
 	if (camel_debug_start("mapi:folder")) {
-		exchange_mapi_debug_property_dump (item_data->properties);
+		exchange_mapi_debug_dump_properties (item_data->conn, item_data->fid, item_data->properties);
 		camel_debug_end();
 	}
 
@@ -103,6 +103,9 @@ fetch_props_to_mail_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 		case PR_MESSAGE_DELIVERY_TIME:
 			delivery_date = (struct FILETIME *) prop_data;
 			break;
+		case PidTagClientSubmitTime:
+			submit_time = (struct FILETIME *) prop_data;
+			break;
 		case PR_MESSAGE_FLAGS:
 			flags = (long *) prop_data;
 			break;
@@ -149,6 +152,13 @@ fetch_props_to_mail_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 
 	if (delivery_date) {
 		item->header.recieved_time = exchange_mapi_util_filetime_to_time_t (delivery_date);
+	} else {
+		item->header.recieved_time = 0;
+	}
+	if (submit_time) {
+		item->header.send_time = exchange_mapi_util_filetime_to_time_t (submit_time);
+	} else {
+		item->header.send_time = 0;
 	}
 
 	if (flags && (*flags & MSGFLAG_READ) != 0)
@@ -181,43 +191,43 @@ fetch_read_item_common_data (MailItem *item, uint32_t propTag, gconstpointer pro
 		} break;
 	/* FIXME : Instead of duping. Use talloc_steal to reuse the memory */
 	case PR_SUBJECT:
-		sv (item->header.subject, utf8tolinux (prop_data));
+		sv (item->header.subject, g_strdup (prop_data));
 		break;
 	case PR_SUBJECT_UNICODE :
 		sv (item->header.subject, g_strdup (prop_data));
 		break;
 	case PR_DISPLAY_TO :
-		sv (item->header.to, utf8tolinux (prop_data));
+		sv (item->header.to, g_strdup (prop_data));
 		break;
 	case PR_DISPLAY_TO_UNICODE :
 		sv (item->header.to, g_strdup (prop_data));
 		break;
 	case PR_DISPLAY_CC:
-		sv (item->header.cc, utf8tolinux (prop_data));
+		sv (item->header.cc, g_strdup (prop_data));
 		break;
 	case PR_DISPLAY_CC_UNICODE:
 		sv (item->header.cc, g_strdup (prop_data));
 		break;
 	case PR_DISPLAY_BCC:
-		sv (item->header.bcc, utf8tolinux (prop_data));
+		sv (item->header.bcc, g_strdup (prop_data));
 		break;
 	case PR_DISPLAY_BCC_UNICODE:
 		sv (item->header.bcc, g_strdup (prop_data));
 		break;
 	case PR_SENT_REPRESENTING_NAME:
-		sv (item->header.from, utf8tolinux (prop_data));
+		sv (item->header.from, g_strdup (prop_data));
 		break;
 	case PR_SENT_REPRESENTING_NAME_UNICODE:
 		sv (item->header.from, g_strdup (prop_data));
 		break;
 	case PR_SENT_REPRESENTING_EMAIL_ADDRESS:
-		sv (item->header.from_email, utf8tolinux (prop_data));
+		sv (item->header.from_email, g_strdup (prop_data));
 		break;
 	case PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE:
 		sv (item->header.from_email, g_strdup (prop_data));
 		break;
 	case PR_SENT_REPRESENTING_ADDRTYPE:
-		sv (item->header.from_type, utf8tolinux (prop_data));
+		sv (item->header.from_type, g_strdup (prop_data));
 		break;
 	case PR_SENT_REPRESENTING_ADDRTYPE_UNICODE:
 		sv (item->header.from_type, g_strdup (prop_data));
@@ -235,7 +245,7 @@ fetch_read_item_common_data (MailItem *item, uint32_t propTag, gconstpointer pro
 		item->header.in_reply_to = g_strdup (prop_data);
 		break;
 	case PR_TRANSPORT_MESSAGE_HEADERS:
-		sv (item->header.transport_headers, utf8tolinux (prop_data));
+		sv (item->header.transport_headers, g_strdup (prop_data));
 		break;
 	case PR_TRANSPORT_MESSAGE_HEADERS_UNICODE:
 		sv (item->header.transport_headers, g_strdup (prop_data));
@@ -263,6 +273,7 @@ mapi_mail_get_item_prop_list (ExchangeMapiConnection *conn, mapi_id_t fid, TALLO
 		PR_MESSAGE_SIZE,
 		PR_MESSAGE_FLAGS,
 		PR_MESSAGE_DELIVERY_TIME,
+		PidTagClientSubmitTime,
 		PR_MSG_EDITOR_FORMAT,
 
 		PR_SUBJECT_UNICODE,
@@ -425,10 +436,7 @@ static void
 mapi_mime_set_msg_headers (ExchangeMapiConnection *conn, CamelMimeMessage *msg, MailItem *item)
 {
 	gchar *temp_str = NULL;
-	time_t recieved_time;
 	CamelInternetAddress *addr = NULL;
-	gint offset = 0;
-	time_t actual_time;
 
 	/* Setting headers from PR_TRANSPORT_MESSAGE_HEADERS */
 	if (item->header.transport_headers) {
@@ -463,7 +471,12 @@ mapi_mime_set_msg_headers (ExchangeMapiConnection *conn, CamelMimeMessage *msg,
 		g_object_unref (parser);
 		g_object_unref (part);
 	} else {
-		recieved_time = item->header.recieved_time;
+		time_t recieved_time, actual_time;
+		gint offset = 0;
+
+		recieved_time = item->header.send_time;
+		if (!recieved_time)
+			recieved_time = item->header.recieved_time;
 		actual_time = camel_header_decode_date (ctime(&recieved_time), &offset);
 		camel_mime_message_set_date (msg, actual_time, offset);
 	}
@@ -531,6 +544,8 @@ mapi_mime_msg_body (MailItem *item, const ExchangeMAPIStream *body)
 		if (item->header.cpid && (body->proptag & 0xFFFF) != PT_UNICODE) {
 			if (item->header.cpid == 20127)
 				buff = g_strdup_printf ("%s; charset=\"us-ascii\"", type);
+			else if (item->header.cpid == 20866)
+				buff = g_strdup_printf ("%s; charset=\"koi8-r\"", type);
 			else if (item->header.cpid >= 28591 && item->header.cpid <= 28599)
 				buff = g_strdup_printf ("%s; charset=\"ISO-8859-%d\"", type, item->header.cpid % 10);
 			else if (item->header.cpid == 28603)
@@ -846,7 +861,7 @@ mapi_mime_classify_attachments (ExchangeMapiConnection *conn, mapi_id_t fid, con
 
 			mem = camel_stream_mem_new ();
 			camel_stream_write (mem, (const gchar *) stream->value->data, stream->value->len, NULL, NULL);
-			camel_stream_reset (mem, NULL);
+			g_seekable_seek (G_SEEKABLE (mem), 0, G_SEEK_SET, NULL, NULL);
 
 			parser = camel_mime_parser_new ();
 			camel_mime_parser_scan_from (parser, FALSE);
@@ -1523,6 +1538,9 @@ mapi_mime_message_to_mail_item (CamelMimeMessage *message, gint32 message_camel_
 	msg_time = camel_mime_message_get_date (message, &msg_time_offset);
 	if (msg_time == CAMEL_MESSAGE_DATE_CURRENT)
 		msg_time = camel_mime_message_get_date_received (message, &msg_time_offset);
+	mail_item_set_time (&item->header.send_time, msg_time, msg_time_offset);
+
+	msg_time = camel_mime_message_get_date_received (message, &msg_time_offset);
 	mail_item_set_time (&item->header.recieved_time, msg_time, msg_time_offset);
 
 	to = camel_mime_message_get_recipients(message, CAMEL_RECIPIENT_TYPE_TO);
@@ -1621,7 +1639,7 @@ mapi_mail_utils_create_item_build_props (ExchangeMapiConnection *conn, mapi_id_t
 	}
 	
 	if (item->pid_name_content_type) {
-		if (!exchange_mapi_utils_add_spropvalue_named_id (conn, fid, mem_ctx, values, n_values, PidNameContentType, item->pid_name_content_type))
+		if (!exchange_mapi_utils_add_spropvalue_namedid (conn, fid, mem_ctx, values, n_values, PidNameContentType, item->pid_name_content_type))
 			return FALSE;
 	}
 
@@ -1652,6 +1670,14 @@ mapi_mail_utils_create_item_build_props (ExchangeMapiConnection *conn, mapi_id_t
 		set_value (PR_MESSAGE_DELIVERY_TIME, &msg_date);
 	}
 
+	if (item->header.send_time != 0) {
+		struct FILETIME msg_date = { 0 };
+
+		exchange_mapi_util_time_t_to_filetime (item->header.send_time, &msg_date);
+
+		set_value (PidTagClientSubmitTime, &msg_date);
+	}
+
 	if (item->header.transport_headers && *item->header.transport_headers)
 		set_value (PR_TRANSPORT_MESSAGE_HEADERS_UNICODE, item->header.transport_headers);
 
