@@ -824,32 +824,30 @@ exchange_crlf_to_lf (const gchar *in)
 }
 
 /**
- * exchange_mapi_util_profiledata_from_camelurl:
+ * exchange_mapi_util_profiledata_from_settings:
  * @empd: destination for profile settings
- * @url: CamelURL with account settings
+ * @settings: a #CamelMapiSettings
  *
  * Sets the members of an ExchangeMapiProfileData instance to
- * reflect the account settings listed in the corresponding
- * CamelURL pointer.
+ * reflect the account settings in @settings.
  *
- * @note: no allocation is done, so do not free the CamelUrl pointer and
- *        the respective underlying pointers until you no longer need the
+ * @note: no allocation is done, so do not finalize @settings and the
+ *        respective underlying pointers until you no longer need the
  *        profile data.
  **/
 void
-exchange_mapi_util_profiledata_from_camelurl (ExchangeMapiProfileData *empd, const CamelURL *url)
+exchange_mapi_util_profiledata_from_settings (ExchangeMapiProfileData *empd, CamelMapiSettings *settings)
 {
-	const gchar *use_ssl = NULL, *use_krb = NULL;
-	CamelURL *promise_its_const = (CamelURL*)url; /* :) */
-	empd->username = promise_its_const->user;
-	empd->server = promise_its_const->host;
+	CamelNetworkSettings *network_settings;
+	CamelNetworkSecurityMethod security_method;
 
-	use_ssl = camel_url_get_param (promise_its_const, "ssl");
-	empd->use_ssl = (use_ssl && g_str_equal (use_ssl, "1"));
-	empd->domain = camel_url_get_param (promise_its_const, "domain");
-	use_krb = camel_url_get_param (promise_its_const, "kerberos");
-	empd->krb_sso = (use_krb && g_str_equal (use_krb, "required"));
-	empd->krb_realm = camel_url_get_param (promise_its_const, "realm");
+	network_settings = CAMEL_NETWORK_SETTINGS (settings);
+	security_method = camel_network_settings_get_security_method (network_settings);
+
+	empd->use_ssl = (security_method != CAMEL_NETWORK_SECURITY_METHOD_NONE);
+	empd->domain = camel_mapi_settings_get_domain (settings);
+	empd->krb_sso = camel_mapi_settings_get_kerberos (settings);
+	empd->krb_realm = camel_mapi_settings_get_realm (settings);
 }
 
 gboolean
@@ -887,8 +885,15 @@ exchange_mapi_util_trigger_krb_auth (const ExchangeMapiProfileData *empd, GError
 	reply = g_dbus_connection_send_message_with_reply_sync (connection, message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, 300 * 1000, NULL, NULL, &local_error);
 	g_free (name);
 
+	if (!local_error && reply) {
+		if (g_dbus_message_to_gerror (reply, &local_error)) {
+			g_object_unref (reply);
+			reply = NULL;
+		}
+	}
+
 	if (local_error) {
-		g_warning ("%s: %s\n", G_STRFUNC, local_error->message);
+		g_dbus_error_strip_remote_error (local_error);
 		g_propagate_error (error, local_error);
 	}
 
@@ -1119,7 +1124,7 @@ exchange_mapi_util_filetime_to_time_t (const struct FILETIME *filetime)
 	NTTIME nt;
 
 	if (!filetime)
-		return (time_t) -1;
+		return (time_t) 0;
 
 	nt = filetime->dwHighDateTime;
 	nt = nt << 32;
